;; Copyright 2009, 2010, 2012, 2013, 2014 Kevin Ryde.

;; my-byte-compile.el is shared by several distributions.
;;
;; my-byte-compile.el is free software; you can redistribute it and/or
;; modify it under the terms of the GNU General Public License as published
;; by the Free Software Foundation; either version 3, or (at your option)
;; any later version.
;;
;; my-byte-compile.el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with my-byte-compile.el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.



;;; Code:

(defun my-byte-compile-filename (filename)
  (unless filename
    (setq filename (file-name-nondirectory (buffer-file-name)))
    ;; no `replace-regexp-in-string' in emacs20
    (if (string-match "\\`d-" filename)
        (setq filename (substring filename 2))))

  (let (dir)
    (if (file-name-directory filename)
        (setq dir (file-name-directory filename))
      (dolist (d (mapcar 'expand-file-name '(".." "../wip" "~/lisp")))
        (when (file-exists-p (expand-file-name filename d))
          (setq dir d))))
    (unless dir
      (error "File %S not found" filename))

    (expand-file-name filename dir)))

(defun my-byte-compile (&optional filename)
  (setq filename (my-byte-compile-filename filename))
  (let (dir command)
    (setq command (format "cd %s; set -x;\n"
                          (shell-quote-argument
                           (file-name-directory filename))))
    (setq command (concat command
                          (format "emacs-checkdoc-batch %s;\n"
                                  (shell-quote-argument filename))))
    (dolist (emacs '("emacs24"
                     "emacs23"
                     ;; "emacs-cvs-b"
                     "emacs22"
                     "xemacs21"
                     "emacs21"
                     "emacs20"))
      (setq command (concat command
                            (format "%s -batch -q -no-site-file -l ~/p/el/bytecomp-simplify.el -f batch-byte-compile %s;\n"
                                    (shell-quote-argument emacs)
                                    (shell-quote-argument
                                     (file-name-nondirectory filename))))))
    (setq command (concat command
                          (format "rm %sc" (shell-quote-argument filename))))

    (compile command)))

(if (locate-library "mm-util")
    (progn
      (autoload 'mm-make-temp-file "mm-util")
      (defalias 'my-byte-compile--make-temp-file 'mm-make-temp-file))
  (require 'poe)
    (defalias 'my-byte-compile--make-temp-file 'make-temp-file))

(defun my-show-autoloads (&optional filename)
  (setq filename (my-byte-compile-filename filename))
  (let ((tempfile (my-byte-compile--make-temp-file
                   "my-byte-compile-" nil ".el"))
        (print-level nil))
    (require 'autoload)
    (find-file tempfile)
    (erase-buffer)
    (generate-file-autoloads filename)
    (save-buffer)
    (byte-compile-file tempfile)
    (delete-file (concat tempfile "c"))
    (delete-file tempfile)))

(provide 'my-byte-compile)

;;; my-byte-compile.el ends here
