;; Copyright 2011, 2012, 2013, 2014 Kevin Ryde.

;; This file is part of weblint-el.
;;
;; weblint-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; weblint-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with weblint-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defvar my-have-weblint-program
  (numberp (condition-case nil ;; error if no such program
               (call-process "weblint"
                             nil ;; input, /dev/null
                             nil ;; output, discard
                             nil ;; display
                             "--version")
             (error nil))))
(message "weblint program available: %S" my-have-weblint-program)

(defun my-run-test ()

  ;; xemacs 21.4.22 in -batch mode segfaults on something to do with window
  ;; size, so dummy out this shrink
  (defun shrink-window-if-larger-than-buffer (&optional window))

  (let ((count   0)
        (tempbuf "*t-weblint-mode--temp*"))
    (switch-to-buffer tempbuf)

    ;; ;;-------------------------------------------------------------------------
    ;; ;; autoload
    ;; 
    ;; (mapcar (lambda (func)
    ;;           (unless (fboundp func)
    ;;             (error "`%S' not autoloaded" func))
    ;;           (setq count (1+ count)))
    ;;         '(weblint weblint-after-save-setup))

    ;;-------------------------------------------------------------------------
    ;; load

    (require 'weblint)


    ;;-------------------------------------------------------------------------
    ;; customize ... maybe

    ;; (save-excursion
    ;;   (customize-variable 'html-mode-hook)
    ;;   (kill-buffer nil))


    ;;-------------------------------------------------------------------------

    (when my-have-weblint-program
      ;; not ".html" or xemacs psgml html-mode tries to start with a template
      (let ((filename "t-weblint.tmp"))
        (condition-case nil (delete-file filename) (error nil))
        (find-file filename)
        (weblint-after-save-setup)
        (insert "<html></html>")
        (save-buffer)

        (unless weblint-output-buffer
          (error "No weblint-output-buffer set after save"))
        
        ;; Emacs 24.1 had a bug where delete-window would segfault under
        ;; -batch.  Switch all windows to tempbuf so the kill-buffer-hook
        ;; setup by weblint-after-save-handler doesn't try to delete any
        ;; windows.
        (walk-windows
         (lambda (window)
           (set-window-buffer window tempbuf)))

        (kill-buffer (current-buffer))
        (delete-file filename)))

    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (unload-feature 'weblint t)

    ;;-------------------------------------------------------------------------

    (bury-buffer)
    (message "  %d tests" count)
    count))
